set :stage, "production" unless variables[:stage]

default_run_options[:shell] = false

case stage
	when "production"
		set :application, "sparkmarble.com"
		set :spin_file, "spin"
		set :ferret_file, "ferret_server -e production"
		set :symlink_location, "insert_symlink"
		set :rails_env, 'production' 
	when "test"
		set :application, "test.sparkmarble.com"
		set :spin_file, "spin_test"
		set :ferret_file, "ferret_server -e test"
		set :symlink_location, "insert_symlink_test"
		set :rails_env, 'test'
	when "development"
		set :application, "dev.sparkmarble.com"
		set :spin_file, "spin_dev"
		set :ferret_file, "ferret_server -e development"
		set :symlink_location, "insert_symlink_dev"
		set :rails_env, 'development'
	else
		raise "unsupported staging environment #{stage}"
end
	

set :user, "mike"
set :repository, "file:///home/mike/repositories/monstapedia.com/trunk"
set :local_repository, "svn+monstapediassh://monstapedia.com/home/mike/repositories/monstapedia.com/trunk"

set :port, 25000
set :deploy_to, "/home/mike/public_html/#{application}"

role :app, application
role :web, application
role :db,  application, :primary => true

set :runner, user

namespace :deploy do
	desc "Custom Tasks for Start/Stop/Restart"
	default_run_options[:shell] = false
	
	task :start_web do
		run "cd #{deploy_to}/current && nohup script/#{spin_file}"
	end
	
	task :stop_web do
		run "cd #{deploy_to}/current && nohup script/process/reaper -a kill"
	end
	
	task :start_ferret do
  		run "ln -nfs #{shared_path}/resources/index #{deploy_to}/current/index"
		run "cd #{deploy_to}/current && nohup script/#{ferret_file} start"
	end
	
	task :stop_ferret do
		run "cd #{deploy_to}/current && nohup script/#{ferret_file} stop"
	end
end

before "deploy:symlink", "deploy:stop_ferret"
before "deploy:symlink", "deploy:stop_web"

after "deploy:restart", "deploy:start_ferret"
after "deploy:restart", "deploy:start_web"
after "deploy", symlink_location
