require 'rubygems'
require 'soap/wsdlDriver'
require 'uuidtools'
require 'rio'
require 'activerecord'
  
@@accessKeyId = "1W7TPN5VE0B9KW723K82"

ActiveRecord::Base.establish_connection(
  :adapter  => "mysql",
  :host => "127.0.0.1",
  :database => "sparkmarble",
  :username => "rubyuser",
  :password => "rubyuser"
)

class Book < ActiveRecord::Base
  has_and_belongs_to_many :subjects
  has_many :similarities, :foreign_key => 'book_id',
                      :class_name => 'SimilarBook',
                      :dependent => :destroy
  has_many :similar_books, :through => :similarities
end

class SimilarBook < ActiveRecord::Base
  belongs_to :similar_book, :class_name => "Book"
end

class Subject < ActiveRecord::Base 
  has_and_belongs_to_many :books
end

def getBook(isbn)
  factory = SOAP::WSDLDriverFactory.new("http://webservices.amazon.com/AWSECommerceService/AWSECommerceService.wsdl")
  service = factory.create_rpc_driver
  soap_params = {
    'AWSAccessKeyId' => @@accessKeyId,
    'Service' => 'AWSECommerceService',
    'Request' => [{
    	'SearchIndex' => 'Books',
  		'IdType' => 'ISBN',
		'ItemId' => isbn,
		'ResponseGroup' => 'ItemAttributes,Images,EditorialReview,Subjects'
    }]
  }

  #http://ecs.amazonaws.com/onca/xml?Service=AWSECommerceService&AWSAccessKeyId=1W7TPN5VE0B9KW723K82&Operation=ItemLookup&IdType=ISBN&ItemId=0764558315&SearchIndex=Books&ResponseGroup=ItemAttributes,Images,EditorialReview,Subjects,Similarities
  service_output = service.ItemLookup(soap_params)
  item = service_output.items.item
  book = create_book(item)
  
  unless book.nil?
	puts "Getting similarities for " + book.book_name
	getAssociations(book)
  end
end
  
def getAssociations(base_book)
  factory = SOAP::WSDLDriverFactory.new("http://webservices.amazon.com/AWSECommerceService/AWSECommerceService.wsdl")
  service = factory.create_rpc_driver
  soap_params = {
    'AWSAccessKeyId' => @@accessKeyId,
    'Service' => 'AWSECommerceService',
    'Request' => [{
    'ItemId' => base_book.ASIN,
    'SimilarityType' => 'Intersection',
    'ResponseGroup' => 'ItemAttributes,Images,EditorialReview,Subjects'
    }]
  }
  service_output = service.SimilarityLookup(soap_params)

  if service_output.items.respond_to?('item')
    service_output.items.item.each do |item|
      book = create_book(item)
      unless book.nil?
        similarity = SimilarBook.find(:first, :conditions => {:book_id => base_book.id, :similar_book_id => book.id })
        if similarity.nil?
          puts "Creating similarity between " + base_book.book_name + " and " + book.book_name
          base_book.similarities.create(:similar_book_id => book.id, :strength => 1)
        else
          puts "Skipping association between " + base_book.book_name + " and " + book.book_name
          #similarity.update_attribute(:strength => similarity.strength + 1)
        end
      end
    end
  end
end

def create_book(item)
  
  puts "Inspecting " + item.aSIN
  if !item.itemAttributes.respond_to?('iSBN')
    puts "Can't find an isbn for this book - skipping"
    return nil
  end
  
  if item.respond_to?('editorialReviews')
    if item.editorialReviews.editorialReview.is_a?(Array)
      description = item.editorialReviews.editorialReview.first.content
    else
      description = item.editorialReviews.editorialReview.content
    end
  elsif item.respond_to?('editorialReview')
    description = item.editorialReview.content
  else
    description = "No description found, we'll fill this in later.  Sorry for the inconvenience."
  end

  book = Book.find_by_isbn(item.itemAttributes.iSBN)
  if book.nil?
    puts "Creating new book " + item.itemAttributes.title
    
    if item.respond_to?('smallImage')
      small_image_name =  UUID.random_create.to_s + '.jpg'
      rio(item.smallImage.uRL) > rio('public/images/books/small/' + small_image_name)
    else
      large_image_name =  'default.jpg'
    end
    
    if item.respond_to?('mediumImage')
      medium_image_name =  UUID.random_create.to_s + '.jpg'
      rio(item.mediumImage.uRL) > rio('public/images/books/medium/' + medium_image_name)
    else
      large_image_name =  'default.jpg'
    end
    
    if item.respond_to?('largeImage')
      large_image_name =  UUID.random_create.to_s + '.jpg'
      rio(item.largeImage.uRL) > rio('public/images/books/large/' +  large_image_name)
    else
      large_image_name =  'default.jpg'
    end

    book = Book.create( :book_name => item.itemAttributes.title, 
      :isbn => item.itemAttributes.iSBN,
      :ASIN => item.aSIN,
      :book_description => description,
      :small_image_name => small_image_name,
      :medium_image_name => medium_image_name,
      :large_image_name => large_image_name)
      
    
    if item.itemAttributes.author.is_a?(Array)
    	authorArray = item.itemAttributes.author
    	authorArray.each do |author|
        	puts "Creating new author " + author
    	end
    else
    	puts "Creating new author " + item.itemAttributes.author
    end		

    if item.respond_to?('subjects')
      if item.subjects.subject.is_a?(Array)
        subjectArray = item.subjects.subject
        subjectArray.each do |subject|
          thisSubject = Subject.find(:first, :conditions => { :subject_name => subject })
          if thisSubject.nil?
            thisSubject = Subject.create(:subject_name => subject )
          end
          book.subjects.push(thisSubject)
        end
      end
    end

    
    book
  else
    puts "This book " + book.book_name + " already exists.  Skipping."
    book
  end
end

getBook(ARGV[0])