require 'rubygems'
require 'soap/wsdlDriver'
require 'uuidtools'
require 'rio'
require 'activerecord'
require 'csv'

#http://ecs.amazonaws.com/onca/xml?Service=AWSECommerceService&AWSAccessKeyId=1W7TPN5VE0B9KW723K82&Operation=ItemLookup&IdType=ISBN&ItemId=0764558315&SearchIndex=Books&ResponseGroup=ItemAttributes,Images,EditorialReview,Subjects,Similarities  
@@accessKeyId = "1W7TPN5VE0B9KW723K82"

ActiveRecord::Base.establish_connection(
  :adapter  => "mysql",
  :host => "127.0.0.1",
  :database => "sparkmarble",
  :username => "rubyuser",
  :password => "rubyuser"
)

class Book < ActiveRecord::Base
  has_many :authors
  has_many :authors_books, :foreign_key => "book_id"
end

class AuthorsBook < ActiveRecord::Base
  set_table_name "authors_books"
  belongs_to :author, :foreign_key => "author_id"
  belongs_to :book, :foreign_key => "book_id"
end

class Author < ActiveRecord::Base
  has_and_belongs_to_many :books
end

def importFile(filename)
	results_file = File.new("results_file.xls", 'w')

	parsed_file = CSV.open(filename, 'r')

	parsed_file.each do |row|
		if row[0] != "Title"
			author = row[1].strip
			keywords = "title: " + row[0].strip + " and author: " + author
			puts "IMPORT: [" + keywords + "]"
			
			new_isbn = getBook(keywords, row[2], row[3])
			if new_isbn == ""
				puts "first import failed.  trying with just book title"
				new_isbn = getBook("title: " + row[0].strip + " and author: " + author.split(",")[0], row[2], row[3])
			end
			puts "RESULT: " + new_isbn
			
			if new_isbn == ""
				result_value = "NOT AVAILABLE"
			else
				result_value = new_isbn
			end
			
			results_file.puts("\"" + result_value + "\",\"" + row[0].strip + "\",\"" + row[1].strip + "\",\"" + row[2].strip + "\",\"" + row[3].strip + "\",\"" + new_isbn + "\"")
			
			sleep_timer = rand(5) + 2
			puts "Sleeping for " + sleep_timer.to_s + " seconds"

			sleep(sleep_timer)
		end
	end
end


def getBook(keywords, fp, atos)

	new_isbn = ""
	
	factory = SOAP::WSDLDriverFactory.new("http://webservices.amazon.com/AWSECommerceService/AWSECommerceService.wsdl")
	service = factory.create_rpc_driver
	soap_params = {
	'AWSAccessKeyId' => @@accessKeyId,
	'Service' => 'AWSECommerceService',
	'Request' => [{
	'SearchIndex' => 'Books',
	'Power' => keywords,
	'ResponseGroup' => 'ItemAttributes,Images,EditorialReview,Subjects'
	}]
	}

	service_output = service.ItemSearch(soap_params)
		if service_output.items.respond_to?('item')
			if service_output.items.item.is_a?(Array)
				service_output.items.item.each do |item|
				puts "Inspecting " + item.aSIN
				this_isbn = create_book(item, fp, atos)
				if this_isbn != ""
					puts "Book Creation Successful!"
					new_isbn = this_isbn
					break
				end
			end
		else
			this_isbn = create_book(service_output.items.item, fp, atos)
			if this_isbn != ""
				puts "Book Creation Successful!"
				new_isbn = this_isbn
			end
		end
	end
	
	return new_isbn
end
  
def getAssociations(base_book)
  factory = SOAP::WSDLDriverFactory.new("http://webservices.amazon.com/AWSECommerceService/AWSECommerceService.wsdl")
  service = factory.create_rpc_driver
  soap_params = {
    'AWSAccessKeyId' => @@accessKeyId,
    'Service' => 'AWSECommerceService',
    'Request' => [{
    'ItemId' => base_book.ASIN,
    'SimilarityType' => 'Intersection',
    'ResponseGroup' => 'ItemAttributes,Images,EditorialReview,Subjects'
    }]
  }
  service_output = service.SimilarityLookup(soap_params)

  if service_output.items.respond_to?('item')
    service_output.items.item.each do |item|
      book = create_book(item)
      unless book.nil?
        similarity = SimilarBook.find(:first, :conditions => {:book_id => base_book.id, :similar_book_id => book.id })
        if similarity.nil?
          puts "Creating similarity between " + base_book.book_name + " and " + book.book_name
          base_book.similarities.create(:similar_book_id => book.id, :strength => 1)
        else
          puts "Skipping association between " + base_book.book_name + " and " + book.book_name
          #similarity.update_attribute(:strength => similarity.strength + 1)
        end
      end
    end
  end
end

def create_book(item, fp, atos)
  
  if !item.itemAttributes.respond_to?('iSBN')
    puts "Can't find an isbn for this book - skipping"
    return ""
  end
 
  book = Book.find_by_isbn(item.itemAttributes.iSBN)
  if book.nil?	
    puts "Creating new book " + item.itemAttributes.title
    
	if item.respond_to?('editorialReviews')
		if item.editorialReviews.editorialReview.is_a?(Array)
			description = item.editorialReviews.editorialReview.first.content
		else
			description = item.editorialReviews.editorialReview.content
		end
	elsif item.respond_to?('editorialReview')
		description = item.editorialReview.content
	else
		description = "No description found, we'll fill this in later.  Sorry for the inconvenience."
	end
    
    if item.respond_to?('smallImage')
      small_image_name =  UUID.random_create.to_s + '.jpg'
      rio(item.smallImage.uRL) > rio('public/images/books/small/' + item.itemAttributes.iSBN + ".jpg")
    else
      small_image_name =  'default.jpg'
    end
    
    if item.respond_to?('mediumImage')
      medium_image_name =  UUID.random_create.to_s + '.jpg'
      rio(item.mediumImage.uRL) > rio('public/images/books/medium/' + item.itemAttributes.iSBN + ".jpg")
    else
      medium_image_name =  'default.jpg'
    end
    
    if item.respond_to?('largeImage')
      large_image_name =  UUID.random_create.to_s + '.jpg'
      rio(item.largeImage.uRL) > rio('public/images/books/large/' +  item.itemAttributes.iSBN + ".jpg")
    else
      large_image_name =  'default.jpg'
    end

    book = Book.create( :book_name => item.itemAttributes.title, 
      :isbn => item.itemAttributes.iSBN,
      :ASIN => item.aSIN,
      :book_description => description,
      :small_image_name => small_image_name,
      :medium_image_name => medium_image_name,
      :large_image_name => large_image_name,
      :FP_level => fp,
      :ATOS_level => atos
      )  
      
    if item.itemAttributes.respond_to?('author')
		if item.itemAttributes.author.is_a?(Array)
			authorArray = item.itemAttributes.author
			authorArray.each do |author|
				create_author(author, book)
			end
		else
			create_author(item.itemAttributes.author, book)
		end	
	end

    book.isbn
  else
    puts "This book " + book.book_name + " already exists.  Skipping."
    book.isbn
  end
end

def create_author(author_name, book)
	author = Author.find_by_name(author_name)
	if author.nil?
		author = Author.create(:name => author_name)
		book.authors_books.create(:author_id => author.id)
		puts "Creating new author " + author.name
	else
		puts "Adding book to existing author " + author.name
		book.authors_books.create(:author_id => author.id)
	end
end


importFile(ARGV[0])