﻿package
{

	import flash.display.*;
	import flash.net.*;
	import flash.events.*;
	import flash.utils.*;
	
	import request.*;
	import response.*;
	import util.*;

	////////////////////////////////////////////////////////////////////////
	//
	////////////////////////////////////////////////////////////////////////
	public class ServerConnection extends Sprite
	{
		public var socket:Socket;

		private var req:Request;
		private var client:Client;
	
		var is_policy_complete:Boolean = false;
		var total_policy_byte_count:int = 0;

		var req_data_buffer:ByteArray = new ByteArray();

		var SIZE_OF_POLICY_DATA:int;

		
		////////////////////////////////////////////////////////////////////////
		//
		//
		//
		////////////////////////////////////////////////////////////////////////
		public function ServerConnection(in_client:Client, in_req:Request)
		{
			client = in_client;
			req = in_req;
			
			if (Client.IS_PROD == true)
				SIZE_OF_POLICY_DATA = 209; 
			else
				SIZE_OF_POLICY_DATA = 212;

			
			socket = new Socket();

			socket.addEventListener(Event.CONNECT, handlerSocketConnect,false);
			socket.addEventListener(IOErrorEvent.IO_ERROR, handlerSocketIOError,false);
			socket.addEventListener(SecurityErrorEvent.SECURITY_ERROR, handlerSocketSecurityError,false);
			socket.addEventListener(ProgressEvent.SOCKET_DATA, handlerSocketData,false);
			socket.addEventListener(Event.CLOSE, handlerSocketDataComplete,false);

			socket.connect(client.getJavaServerAddress(), client.getJavaServerPort());
			
		}


		////////////////////////////////////////////////////////////////////////
		//
		//
		////////////////////////////////////////////////////////////////////////
		public function shutdown():void
		{
			socket.removeEventListener(Event.CLOSE, handlerSocketDataComplete);
			socket.removeEventListener(ProgressEvent.SOCKET_DATA, handlerSocketData);
			socket.removeEventListener(SecurityErrorEvent.SECURITY_ERROR, handlerSocketSecurityError);
			socket.removeEventListener(IOErrorEvent.IO_ERROR, handlerSocketIOError);
			socket.removeEventListener(Event.CONNECT, handlerSocketConnect);
			
			socket.close();

		}


		////////////////////////////////////////////////////////////////////////
		//
		////////////////////////////////////////////////////////////////////////
		private function handlerSocketConnect(event:Event):void
		{
			//send the request to the server
			send(req.toBytes());
		}


		////////////////////////////////////////////////////////////////////////
		//
		////////////////////////////////////////////////////////////////////////
		private function handlerSocketIOError(event:IOErrorEvent):void
		{
			UtilLog.write("io error: " + event);
			
			navigateToURL(new URLRequest(Client.URL_ERROR_PAGE),"_self");
		
			shutdown();
		
		}
		
		
		////////////////////////////////////////////////////////////////////////
		//
		////////////////////////////////////////////////////////////////////////
		private function handlerSocketSecurityError(event:SecurityErrorEvent):void
		{	 
			UtilLog.write("security error: " + event);
			
			navigateToURL(new URLRequest(Client.URL_ERROR_PAGE),"_self");
		
			shutdown();
		}


		////////////////////////////////////////////////////////////////////////
		//
		////////////////////////////////////////////////////////////////////////
		private function handlerSocketData(event:ProgressEvent):void 
		{
			var idx_start:int = req_data_buffer.length;
			
			socket.readBytes(req_data_buffer,req_data_buffer.length,socket.bytesAvailable);

			UtilCipher.decrypt(req_data_buffer,idx_start,req_data_buffer.length - idx_start);

		}

		////////////////////////////////////////////////////////////////////////
		//
		////////////////////////////////////////////////////////////////////////
		private function handlerSocketDataComplete(event:Event):void 
		{
			var buf:ByteArray = new ByteArray();

			req_data_buffer.readBytes(buf);

			var rsp:Response;
			
			try
			{
				rsp = Response.factory(client,buf);
				client.addToResponseQueue(rsp)
			}
			catch(e:Error)
			{
				UtilLog.write("unable to create response obj:" + buf);
			}
			
			shutdown();
		}

		////////////////////////////////////////////////////////////////////////
		//
		//
		////////////////////////////////////////////////////////////////////////
		public function send(data:ByteArray):ByteArray
		{
			var enc_data:ByteArray;
			
			enc_data = UtilCipher.encrypt(data);
			
			var i:int;

			if (Client.IS_DEBUG)
			{
				var the_data:String = "sending data(";
				
				for(i=0;i<data.length;i++)
				{
					the_data += data[i] + ",";
				}

				the_data += ") (note: showing as unecrypted)";

				UtilLog.write(the_data);
			}
			
			try
			{
				socket.writeBytes(enc_data);
				socket.writeByte(-1);
				socket.flush();
			}
			catch(err:Error)
			{
				UtilLog.write("send failed: server might be down");
				
				navigateToURL(new URLRequest(Client.URL_ERROR_PAGE),"_self");
			}
			
			return enc_data;
		}  
		
	}
	
}
