
package base;

import java.net.*;
import java.io.*;
import java.util.*;


import config.*;
import util.*;

//////////////////////////////////////////////////////////////////////
//
//
//////////////////////////////////////////////////////////////////////
public class Server
{
	private static String VERSION = "Server.1.0";
	
	ServerSocket server;

	Vector<ClientHandler> list_thread;
	

	//////////////////////////////////////////////////////////////////////
	//
	//////////////////////////////////////////////////////////////////////
	public static void main(String[] args)
	{
		try
		{
			new Server();
		}
		catch(Exception e)
		{
			UtilLog.write("server has exited with error: " + e.getMessage());
			e.printStackTrace();
		}
	}

	
	//////////////////////////////////////////////////////////////////////
	//
	//////////////////////////////////////////////////////////////////////
    public Server() throws Exception
    {
       	init();
          	
    	UtilEmail.postMail(Config.email_for_logging, "spark server started","spark server started",Config.email_for_logging);
    	       	
    	process();

    	String str ="spark server stopped";
    	UtilLog.write(str);
    	UtilEmail.postMail(Config.email_for_logging, str,str,Config.email_for_logging);
    	
    }


    
	//////////////////////////////////////////////////////////////////////
	//
	//////////////////////////////////////////////////////////////////////
    private void init() throws Exception
    {
    	Config.setFile("../conf/server.cfg");
    	UtilLog.clear();
   	
    	UtilLog.write("spark server starting on port " + Config.port);
    	server = new ServerSocket(Config.port);

    	UtilLog.write("supporting " + Config.max_threads + " threads");
    	list_thread = new Vector<ClientHandler>(Config.max_threads,1);    	

    	UtilLog.write("version " + VERSION);
    	
    }

	//////////////////////////////////////////////////////////////////////
	//
	//////////////////////////////////////////////////////////////////////
    private void process() throws Exception 
    {
    	while(true)
    	{
    		try
    		{
    			Socket socket = server.accept();

    			if (list_thread.size() < Config.max_threads)
    			{
    				UtilLog.write("    starting thread for socket " + socket.getInetAddress());
    		    	
    				ClientHandler thread = new ClientHandler(this, socket);
    			
    				list_thread.add(thread);
    				   	  	    
    				thread.start();
    			}
    			else
    			{
    				UtilLog.write("    ERROR: unable to accept socket from " + socket.getInetAddress() + " due to max threads");

        	    	UtilEmail.postMail(Config.email_for_logging, "max threads reached","",Config.email_for_logging);
    			}
    			
    		}
    		catch (IOException e)
    		{
    			e.printStackTrace();
    			
    	    	UtilEmail.postMail(Config.email_for_logging, "server exception",e.getMessage(),Config.email_for_logging);

    		}
   		}

    }


	//////////////////////////////////////////////////////////////////////
	//
	//////////////////////////////////////////////////////////////////////
    public void removeClientHandler(ClientHandler ch)
    {
    	list_thread.remove(ch);

    	UtilLog.write("    ending thread. (" + list_thread.size() + ")");
		    
    }
}

