
package request;

import util.UtilByte;
import util.UtilLog;



//////////////////////////////////////////////////////////////////////
//
//
//////////////////////////////////////////////////////////////////////
public abstract class Request
{
	int id;

	byte[] data;

	public static final int ID_TEST 		 	   = 1;
	public static final int ID_GET_APP_INFO  	   = 2;
	public static final int ID_GET_QUIZ_QUESTIONS  = 3;
	public static final int ID_SEND_QUIZ_RESULTS   = 4;
	public static final int ID_GET_EMAILS		   = 5;
	public static final int ID_LOGIN			   = 6;
	public static final int ID_SEND_CLIENT_INFO	   = 7;

	//see comments in client
	public static final int ID_DO_NOT_USE_1			= 54;


	//the reason this id has a value of 60 is because the number 60 is the
	//decimal ascii value for the '<' character.
	//this particular message comes from the client - but is built by the flash player
	//and not the spark client code. the message looks like this:
	//<policy-file-request/>
	public static final int ID_HANDLE_POLICY_FILE = 60;


	//////////////////////////////////////////////////////////////////////
	//
	//////////////////////////////////////////////////////////////////////
    public Request(byte[] in_data) 
    {    	
    	data = in_data;
    }

	//////////////////////////////////////////////////////////////////////
	//
	//////////////////////////////////////////////////////////////////////
    public static boolean isValid(byte data) throws Exception
    {    
    	if (data == ID_TEST)
    		return true;
    	if (data == ID_GET_APP_INFO)
    		return true;
    	if (data == ID_GET_QUIZ_QUESTIONS)
    		return true;
    	if (data == ID_SEND_QUIZ_RESULTS)
    		return true;
    	if (data == ID_GET_EMAILS)
    		return true;
    	if (data == ID_LOGIN)
    		return true;
    	if (data == ID_SEND_CLIENT_INFO)
    		return true;
    	if (data == ID_HANDLE_POLICY_FILE)
    		return true;
    	   	
    	return false;
    }
	
	//////////////////////////////////////////////////////////////////////
	//
	//////////////////////////////////////////////////////////////////////
    public static Request factory(byte[] in_data) throws Exception
    {    
    	Request req;
    	byte req_id;
    	    	
    	req_id = getId(in_data); 
    	
    	if (req_id == ID_TEST)
    		req = new RequestTest(in_data);
    	else if (req_id == ID_GET_APP_INFO)
    		req = new RequestGetAppInfo(in_data);
    	else if (req_id == ID_GET_QUIZ_QUESTIONS)
    		req = new RequestGetQuizQuestions(in_data);
    	else if (req_id == ID_HANDLE_POLICY_FILE)
    		req = new RequestHandlePolicyFile(in_data);
    	else if (req_id == ID_SEND_QUIZ_RESULTS)
    		req = new RequestSendQuizResults(in_data);
    	else if (req_id == ID_GET_EMAILS)
    		req = new RequestGetEmails(in_data);
    	else if (req_id == ID_LOGIN)
    		req = new RequestLogin(in_data);
    	else if (req_id == ID_SEND_CLIENT_INFO)
    		req = new RequestSendClientInfo(in_data);
    	else
    	{
			UtilLog.write("Dump for unsupported request:" + UtilByte.byteArrayToString(in_data) + ":");
    		throw new Exception("        ERROR: unsupported request type " + req_id);
    	}

    	return req;
    }

	//////////////////////////////////////////////////////////////////////
	//
	//////////////////////////////////////////////////////////////////////
    private static byte getId(byte[] byte_req) 
    {   
    	return byte_req[0];    	
    }


	//////////////////////////////////////////////////////////////////////
	//
	//////////////////////////////////////////////////////////////////////
    public abstract byte[] process() throws Exception;

  
}

